@@ -60,13 +60,13 @@ int BinaryGrammar::parse_grammar(std::istream& input) {
 		std::cerr << "Error: Error reading first 4 bytes from grammar!" << std::endl;
 		CG3Quit(1);
 	}
-	if (cbuffers[0][0] != 'C' || cbuffers[0][1] != 'G' || cbuffers[0][2] != '3' || cbuffers[0][3] != 'B') {
+	if (!is_cg3b(cbuffers[0])) {
 		u_fprintf(ux_stderr, "Error: Grammar does not begin with magic bytes - cannot load as binary!\n");
 		CG3Quit(1);
 	}
 
 	fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-	u32tmp = (uint32_t)ntohl(u32tmp);
+	u32tmp = ntoh32(u32tmp);
 	if (u32tmp <= 10297) {
 		if (verbosity >= 1) {
 			u_fprintf(ux_stderr, "Warning: Grammar revision is %u, but current format is %u or later. Please recompile the binary grammar with latest CG-3.\n", u32tmp, CG3_FEATURE_REV);
@@ -87,7 +87,7 @@ int BinaryGrammar::parse_grammar(std::istream& input) {
 	grammar->is_binary = true;
 
 	fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-	fields = (uint32_t)ntohl(u32tmp);
+	fields = ntoh32(u32tmp);
 
 	grammar->has_dep = (fields & (1 << 0)) != 0;
 	grammar->sub_readings_ltr = (fields & (1 << 2)) != 0;
@@ -98,7 +98,7 @@ int BinaryGrammar::parse_grammar(std::istream& input) {
 	if (fields & (1 << 1)) {
 		ucnv_reset(conv);
 		fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-		u32tmp = (uint32_t)ntohl(u32tmp);
+		u32tmp = ntoh32(u32tmp);
 		fread_throw(&cbuffers[0][0], 1, u32tmp, input);
 		i32tmp = ucnv_toUChars(conv, &grammar->mapping_prefix, 1, &cbuffers[0][0], u32tmp, &err);
 	}
@@ -110,7 +110,7 @@ int BinaryGrammar::parse_grammar(std::istream& input) {
 	u32tmp = 0;
 	if (fields & (1 << 3)) {
 		fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-		u32tmp = (uint32_t)ntohl(u32tmp);
+		u32tmp = ntoh32(u32tmp);
 	}
 	uint32_t num_single_tags = u32tmp;
 	grammar->single_tags_list.resize(num_single_tags);
@@ -120,32 +120,32 @@ int BinaryGrammar::parse_grammar(std::istream& input) {
 
 		uint32_t fields = 0;
 		fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-		fields = (uint32_t)ntohl(u32tmp);
+		fields = ntoh32(u32tmp);
 
 		if (fields & (1 << 0)) {
 			fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-			t->number = (uint32_t)ntohl(u32tmp);
+			t->number = ntoh32(u32tmp);
 		}
 		if (fields & (1 << 1)) {
 			fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-			t->hash = (uint32_t)ntohl(u32tmp);
+			t->hash = ntoh32(u32tmp);
 		}
 		if (fields & (1 << 2)) {
 			fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-			t->plain_hash = (uint32_t)ntohl(u32tmp);
+			t->plain_hash = ntoh32(u32tmp);
 		}
 		if (fields & (1 << 3)) {
 			fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-			t->seed = (uint32_t)ntohl(u32tmp);
+			t->seed = ntoh32(u32tmp);
 		}
 		if (fields & (1 << 4)) {
 			fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-			t->type = (uint32_t)ntohl(u32tmp);
+			t->type = ntoh32(u32tmp);
 		}
 
 		if (fields & (1 << 5)) {
 			fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-			t->comparison_hash = (uint32_t)ntohl(u32tmp);
+			t->comparison_hash = ntoh32(u32tmp);
 		}
 		if (fields & (1 << 6)) {
 			fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
@@ -172,7 +172,7 @@ int BinaryGrammar::parse_grammar(std::istream& input) {
 
 		if (fields & (1 << 8)) {
 			fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-			u32tmp = (uint32_t)ntohl(u32tmp);
+			u32tmp = ntoh32(u32tmp);
 			if (u32tmp) {
 				ucnv_reset(conv);
 				fread_throw(&cbuffers[0][0], 1, u32tmp, input);
@@ -183,7 +183,7 @@ int BinaryGrammar::parse_grammar(std::istream& input) {
 
 		if (fields & (1 << 9)) {
 			fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-			u32tmp = (uint32_t)ntohl(u32tmp);
+			u32tmp = ntoh32(u32tmp);
 			if (u32tmp) {
 				ucnv_reset(conv);
 				fread_throw(&cbuffers[0][0], 1, u32tmp, input);
@@ -207,24 +207,24 @@ int BinaryGrammar::parse_grammar(std::istream& input) {
 
 		if (fields & (1 << 10)) {
 			fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-			uint32_t num = (uint32_t)ntohl(u32tmp);
+			auto num = ntoh32(u32tmp);
 			t->allocateVsSets();
 			t->vs_sets->reserve(num);
 			tag_varsets[t->number].reserve(num);
 			for (size_t i = 0; i < num; ++i) {
 				fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-				u32tmp = (uint32_t)ntohl(u32tmp);
+				u32tmp = ntoh32(u32tmp);
 				tag_varsets[t->number].push_back(u32tmp);
 			}
 		}
 		if (fields & (1 << 11)) {
 			fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-			uint32_t num = (uint32_t)ntohl(u32tmp);
+			auto num = ntoh32(u32tmp);
 			t->allocateVsNames();
 			t->vs_names->reserve(num);
 			for (size_t i = 0; i < num; ++i) {
 				fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-				u32tmp = (uint32_t)ntohl(u32tmp);
+				u32tmp = ntoh32(u32tmp);
 				if (u32tmp) {
 					ucnv_reset(conv);
 					fread_throw(&cbuffers[0][0], 1, u32tmp, input);
@@ -245,38 +245,38 @@ int BinaryGrammar::parse_grammar(std::istream& input) {
 	u32tmp = 0;
 	if (fields & (1 << 4)) {
 		fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-		u32tmp = (uint32_t)ntohl(u32tmp);
+		u32tmp = ntoh32(u32tmp);
 	}
 	uint32_t num_remaps = u32tmp;
 	for (uint32_t i = 0; i < num_remaps; ++i) {
 		fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-		u32tmp = (uint32_t)ntohl(u32tmp);
+		u32tmp = ntoh32(u32tmp);
 		grammar->reopen_mappings.insert(u32tmp);
 	}
 
 	u32tmp = 0;
 	if (fields & (1 << 5)) {
 		fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-		u32tmp = (uint32_t)ntohl(u32tmp);
+		u32tmp = ntoh32(u32tmp);
 	}
 	uint32_t num_pref_targets = u32tmp;
 	for (uint32_t i = 0; i < num_pref_targets; i++) {
 		fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-		u32tmp = (uint32_t)ntohl(u32tmp);
+		u32tmp = ntoh32(u32tmp);
 		grammar->preferred_targets.push_back(u32tmp);
 	}
 
 	u32tmp = 0;
 	if (fields & (1 << 6)) {
 		fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-		u32tmp = (uint32_t)ntohl(u32tmp);
+		u32tmp = ntoh32(u32tmp);
 	}
 	uint32_t num_par_pairs = u32tmp;
 	for (uint32_t i = 0; i < num_par_pairs; i++) {
 		fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-		uint32_t left = (uint32_t)ntohl(u32tmp);
+		uint32_t left = ntoh32(u32tmp);
 		fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-		uint32_t right = (uint32_t)ntohl(u32tmp);
+		uint32_t right = ntoh32(u32tmp);
 		grammar->parentheses[left] = right;
 		grammar->parentheses_reverse[right] = left;
 	}
@@ -284,21 +284,21 @@ int BinaryGrammar::parse_grammar(std::istream& input) {
 	u32tmp = 0;
 	if (fields & (1 << 7)) {
 		fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-		u32tmp = (uint32_t)ntohl(u32tmp);
+		u32tmp = ntoh32(u32tmp);
 	}
 	uint32_t num_par_anchors = u32tmp;
 	for (uint32_t i = 0; i < num_par_anchors; i++) {
 		fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-		uint32_t left = (uint32_t)ntohl(u32tmp);
+		uint32_t left = ntoh32(u32tmp);
 		fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-		uint32_t right = (uint32_t)ntohl(u32tmp);
+		uint32_t right = ntoh32(u32tmp);
 		grammar->anchors[left] = right;
 	}
 
 	u32tmp = 0;
 	if (fields & (1 << 8)) {
 		fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-		u32tmp = (uint32_t)ntohl(u32tmp);
+		u32tmp = ntoh32(u32tmp);
 	}
 	uint32_t num_sets = u32tmp;
 	grammar->sets_list.resize(num_sets);
@@ -307,11 +307,11 @@ int BinaryGrammar::parse_grammar(std::istream& input) {
 
 		uint32_t fields = 0;
 		fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-		fields = (uint32_t)ntohl(u32tmp);
+		fields = ntoh32(u32tmp);
 
 		if (fields & (1 << 0)) {
 			fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-			s->number = (uint32_t)ntohl(u32tmp);
+			s->number = ntoh32(u32tmp);
 		}
 		// Field 1 is unused
 		if (fields & (1 << 2)) {
@@ -321,39 +321,39 @@ int BinaryGrammar::parse_grammar(std::istream& input) {
 
 		if (fields & (1 << 3)) {
 			fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-			u32tmp = (uint32_t)ntohl(u32tmp);
+			u32tmp = ntoh32(u32tmp);
 			if (u32tmp) {
 				trie_unserialize(s->trie, input, *grammar, u32tmp);
 			}
 			fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-			u32tmp = (uint32_t)ntohl(u32tmp);
+			u32tmp = ntoh32(u32tmp);
 			if (u32tmp) {
 				trie_unserialize(s->trie_special, input, *grammar, u32tmp);
 			}
 		}
 		if (fields & (1 << 4)) {
 			fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-			u32tmp = (uint32_t)ntohl(u32tmp);
+			u32tmp = ntoh32(u32tmp);
 			uint32_t num_set_ops = u32tmp;
 			for (uint32_t j = 0; j < num_set_ops; j++) {
 				fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-				u32tmp = (uint32_t)ntohl(u32tmp);
+				u32tmp = ntoh32(u32tmp);
 				s->set_ops.push_back(u32tmp);
 			}
 		}
 		if (fields & (1 << 5)) {
 			fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-			u32tmp = (uint32_t)ntohl(u32tmp);
+			u32tmp = ntoh32(u32tmp);
 			uint32_t num_sets = u32tmp;
 			for (uint32_t j = 0; j < num_sets; j++) {
 				fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-				u32tmp = (uint32_t)ntohl(u32tmp);
+				u32tmp = ntoh32(u32tmp);
 				s->sets.push_back(u32tmp);
 			}
 		}
 		if (fields & (1 << 6)) {
 			fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-			u32tmp = (uint32_t)ntohl(u32tmp);
+			u32tmp = ntoh32(u32tmp);
 			if (u32tmp) {
 				ucnv_reset(conv);
 				fread_throw(&cbuffers[0][0], 1, u32tmp, input);
@@ -375,20 +375,20 @@ int BinaryGrammar::parse_grammar(std::istream& input) {
 
 	if (fields & (1 << 9)) {
 		fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-		u32tmp = (uint32_t)ntohl(u32tmp);
+		u32tmp = ntoh32(u32tmp);
 		grammar->delimiters = grammar->sets_list[u32tmp];
 	}
 
 	if (fields & (1 << 10)) {
 		fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-		u32tmp = (uint32_t)ntohl(u32tmp);
+		u32tmp = ntoh32(u32tmp);
 		grammar->soft_delimiters = grammar->sets_list[u32tmp];
 	}
 
 	u32tmp = 0;
 	if (fields & (1 << 11)) {
 		fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-		u32tmp = (uint32_t)ntohl(u32tmp);
+		u32tmp = ntoh32(u32tmp);
 	}
 	uint32_t num_contexts = u32tmp;
 	for (uint32_t i = 0; i < num_contexts; i++) {
@@ -399,7 +399,7 @@ int BinaryGrammar::parse_grammar(std::istream& input) {
 	u32tmp = 0;
 	if (fields & (1 << 12)) {
 		fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-		u32tmp = (uint32_t)ntohl(u32tmp);
+		u32tmp = ntoh32(u32tmp);
 	}
 	uint32_t num_rules = u32tmp;
 	grammar->rule_by_number.resize(num_rules);
@@ -408,7 +408,7 @@ int BinaryGrammar::parse_grammar(std::istream& input) {
 
 		uint32_t fields = 0;
 		fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-		fields = (uint32_t)ntohl(u32tmp);
+		fields = ntoh32(u32tmp);
 
 		if (fields & (1 << 0)) {
 			fread_throw(&i32tmp, sizeof(int32_t), 1, input);
@@ -420,15 +420,15 @@ int BinaryGrammar::parse_grammar(std::istream& input) {
 		}
 		if (fields & (1 << 2)) {
 			fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-			r->line = (uint32_t)ntohl(u32tmp);
+			r->line = ntoh32(u32tmp);
 		}
 		if (fields & (1 << 3)) {
 			fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-			r->flags = (uint32_t)ntohl(u32tmp);
+			r->flags = ntoh32(u32tmp);
 		}
 		if (fields & (1 << 4)) {
 			fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-			u32tmp = (uint32_t)ntohl(u32tmp);
+			u32tmp = ntoh32(u32tmp);
 			if (u32tmp) {
 				ucnv_reset(conv);
 				fread_throw(&cbuffers[0][0], 1, u32tmp, input);
@@ -438,23 +438,23 @@ int BinaryGrammar::parse_grammar(std::istream& input) {
 		}
 		if (fields & (1 << 5)) {
 			fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-			r->target = (uint32_t)ntohl(u32tmp);
+			r->target = ntoh32(u32tmp);
 		}
 		if (fields & (1 << 6)) {
 			fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-			r->wordform = grammar->single_tags_list[(uint32_t)ntohl(u32tmp)];
+			r->wordform = grammar->single_tags_list[ntoh32(u32tmp)];
 		}
 		if (fields & (1 << 7)) {
 			fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-			r->varname = (uint32_t)ntohl(u32tmp);
+			r->varname = ntoh32(u32tmp);
 		}
 		if (fields & (1 << 8)) {
 			fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-			r->varvalue = (uint32_t)ntohl(u32tmp);
+			r->varvalue = ntoh32(u32tmp);
 		}
 		if (fields & (1 << 9)) {
 			fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-			u32tmp = (uint32_t)ntohl(u32tmp);
+			u32tmp = ntoh32(u32tmp);
 			int32_t v = u32tmp;
 			if (u32tmp & (1 << 31)) {
 				u32tmp &= ~(1 << 31);
@@ -465,47 +465,47 @@ int BinaryGrammar::parse_grammar(std::istream& input) {
 		}
 		if (fields & (1 << 10)) {
 			fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-			r->childset1 = (uint32_t)ntohl(u32tmp);
+			r->childset1 = ntoh32(u32tmp);
 		}
 		if (fields & (1 << 11)) {
 			fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-			r->childset2 = (uint32_t)ntohl(u32tmp);
+			r->childset2 = ntoh32(u32tmp);
 		}
 		if (fields & (1 << 12)) {
 			fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-			r->maplist = grammar->sets_list[(uint32_t)ntohl(u32tmp)];
+			r->maplist = grammar->sets_list[ntoh32(u32tmp)];
 		}
 		if (fields & (1 << 13)) {
 			fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-			r->sublist = grammar->sets_list[(uint32_t)ntohl(u32tmp)];
+			r->sublist = grammar->sets_list[ntoh32(u32tmp)];
 		}
 		if (fields & (1 << 14)) {
 			fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-			r->number = (uint32_t)ntohl(u32tmp);
+			r->number = ntoh32(u32tmp);
 		}
 
 		fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-		u32tmp = (uint32_t)ntohl(u32tmp);
+		u32tmp = ntoh32(u32tmp);
 		if (u32tmp) {
 			r->dep_target = grammar->contexts[u32tmp];
 		}
 
 		fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-		u32tmp = (uint32_t)ntohl(u32tmp);
+		u32tmp = ntoh32(u32tmp);
 		uint32_t num_dep_tests = u32tmp;
 		for (uint32_t j = 0; j < num_dep_tests; j++) {
 			fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-			u32tmp = (uint32_t)ntohl(u32tmp);
+			u32tmp = ntoh32(u32tmp);
 			ContextualTest* t = grammar->contexts[u32tmp];
 			r->addContextualTest(t, r->dep_tests);
 		}
 
 		fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-		u32tmp = (uint32_t)ntohl(u32tmp);
+		u32tmp = ntoh32(u32tmp);
 		uint32_t num_tests = u32tmp;
 		for (uint32_t j = 0; j < num_tests; j++) {
 			fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-			u32tmp = (uint32_t)ntohl(u32tmp);
+			u32tmp = ntoh32(u32tmp);
 			ContextualTest* t = grammar->contexts[u32tmp];
 			r->addContextualTest(t, r->tests);
 		}
@@ -537,15 +537,15 @@ ContextualTest* BinaryGrammar::readContextualTest(std::istream& input) {
 	uint32_t tmpl = 0;
 
 	fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-	fields = (uint32_t)ntohl(u32tmp);
+	fields = ntoh32(u32tmp);
 
 	if (fields & (1 << 0)) {
 		fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-		t->hash = (uint32_t)ntohl(u32tmp);
+		t->hash = ntoh32(u32tmp);
 	}
 	if (fields & (1 << 1)) {
 		fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-		t->pos = (uint32_t)ntohl(u32tmp);
+		t->pos = ntoh32(u32tmp);
 		if (t->pos & POS_64BIT) {
 			fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
 			t->pos |= ((uint64_t)ntohl(u32tmp)) << 32;
@@ -557,28 +557,28 @@ ContextualTest* BinaryGrammar::readContextualTest(std::istream& input) {
 	}
 	if (fields & (1 << 3)) {
 		fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-		tmpl = (uint32_t)ntohl(u32tmp);
+		tmpl = ntoh32(u32tmp);
 		deferred_tmpls[t] = tmpl;
 	}
 	if (fields & (1 << 4)) {
 		fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-		t->target = (uint32_t)ntohl(u32tmp);
+		t->target = ntoh32(u32tmp);
 	}
 	if (fields & (1 << 5)) {
 		fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-		t->line = (uint32_t)ntohl(u32tmp);
+		t->line = ntoh32(u32tmp);
 	}
 	if (fields & (1 << 6)) {
 		fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-		t->relation = (uint32_t)ntohl(u32tmp);
+		t->relation = ntoh32(u32tmp);
 	}
 	if (fields & (1 << 7)) {
 		fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-		t->barrier = (uint32_t)ntohl(u32tmp);
+		t->barrier = ntoh32(u32tmp);
 	}
 	if (fields & (1 << 8)) {
 		fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-		t->cbarrier = (uint32_t)ntohl(u32tmp);
+		t->cbarrier = ntoh32(u32tmp);
 	}
 	if (fields & (1 << 9)) {
 		fread_throw(&i32tmp, sizeof(int32_t), 1, input);
@@ -586,16 +586,16 @@ ContextualTest* BinaryGrammar::readContextualTest(std::istream& input) {
 	}
 	if (fields & (1 << 10)) {
 		fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-		uint32_t num_ors = (uint32_t)ntohl(u32tmp);
+		uint32_t num_ors = ntoh32(u32tmp);
 		for (uint32_t i = 0; i < num_ors; ++i) {
 			fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-			u32tmp = (uint32_t)ntohl(u32tmp);
+			u32tmp = ntoh32(u32tmp);
 			deferred_ors[t].push_back(u32tmp);
 		}
 	}
 	if (fields & (1 << 11)) {
 		fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
-		u32tmp = (uint32_t)ntohl(u32tmp);
+		u32tmp = ntoh32(u32tmp);
 		t->linked = grammar->contexts[u32tmp];
 	}
 
