@@ -108,14 +108,14 @@ void MatxinApplicator::runGrammarOnText(std::istream& input, std::ostream& outpu
 
 	uint32_t resetAfter = ((num_windows + 4) * 2 + 1);
 
-	begintag = addTag(stringbits[S_BEGINTAG].getTerminatedBuffer())->hash; // Beginning of sentence tag
-	endtag = addTag(stringbits[S_ENDTAG].getTerminatedBuffer())->hash;     // End of sentence tag
+	begintag = addTag(stringbits[S_BEGINTAG])->hash; // Beginning of sentence tag
+	endtag = addTag(stringbits[S_ENDTAG])->hash;     // End of sentence tag
 
-	SingleWindow* cSWindow = 0; // Current single window (Cohort frame)
-	Cohort* cCohort = 0;        // Current cohort
-	Reading* cReading = 0;      // Current reading
+	SingleWindow* cSWindow = nullptr; // Current single window (Cohort frame)
+	Cohort* cCohort = nullptr;        // Current cohort
+	Reading* cReading = nullptr;      // Current reading
 
-	SingleWindow* lSWindow = 0; // Left hand single window
+	SingleWindow* lSWindow = nullptr; // Left hand single window
 
 	gWindow->window_span = num_windows;
 	gtimer = getticks();
@@ -185,8 +185,8 @@ void MatxinApplicator::runGrammarOnText(std::istream& input, std::ostream& outpu
 
 				cSWindow->appendCohort(cCohort);
 				lSWindow = cSWindow;
-				cSWindow = 0;
-				cCohort = 0;
+				cSWindow = nullptr;
+				cCohort = nullptr;
 				numCohorts++;
 			} // end >= soft_limit
 			if (cCohort && (cSWindow->cohorts.size() >= hard_limit || (grammar->delimiters && doesSetMatchCohortNormal(*cCohort, grammar->delimiters->number)))) {
@@ -200,8 +200,8 @@ void MatxinApplicator::runGrammarOnText(std::istream& input, std::ostream& outpu
 
 				cSWindow->appendCohort(cCohort);
 				lSWindow = cSWindow;
-				cSWindow = 0;
-				cCohort = 0;
+				cSWindow = nullptr;
+				cCohort = nullptr;
 				numCohorts++;
 			} // end >= hard_limit
 			// If we don't have a current window, create one
@@ -225,7 +225,7 @@ void MatxinApplicator::runGrammarOnText(std::istream& input, std::ostream& outpu
 				lSWindow = cSWindow;
 				lSWindow->text = firstblank;
 				firstblank.clear();
-				cCohort = 0;
+				cCohort = nullptr;
 				numWindows++;
 			} // created at least one cSWindow by now
 
@@ -274,7 +274,7 @@ void MatxinApplicator::runGrammarOnText(std::istream& input, std::ostream& outpu
 
 			// We're now at the beginning of the readings
 			UString current_reading;
-			Reading* cReading = 0;
+			Reading* cReading = nullptr;
 
 			// Handle the static reading of ^estació<n><f><sg>/season<n><sg>/station<n><sg>$
 			// Gobble up all <tags> until the first / or $ and stuff them in the static reading
@@ -339,7 +339,7 @@ void MatxinApplicator::runGrammarOnText(std::istream& input, std::ostream& outpu
 				}
 
 				if (inchar == '/') { // Reached end of reading
-					Reading* cReading = 0;
+					Reading* cReading = nullptr;
 					cReading = alloc_reading(cCohort);
 
 					addTagToReading(*cReading, cCohort->wordform);
@@ -382,9 +382,9 @@ void MatxinApplicator::runGrammarOnText(std::istream& input, std::ostream& outpu
 		for (auto iter : cCohort->readings) {
 			addTagToReading(*iter, endtag);
 		}
-		cReading = 0;
-		cCohort = 0;
-		cSWindow = 0;
+		cReading = nullptr;
+		cCohort = nullptr;
+		cSWindow = nullptr;
 	}
 
 	// Run the grammar & print results
