@@ -23,7 +23,7 @@
 #ifndef c6d28b7452ec699b_OPTIONS_H
 #define c6d28b7452ec699b_OPTIONS_H
 
-#include <uoptions.h>
+#include <uoptions.hpp>
 
 namespace Options {
 enum OPTIONS {
@@ -60,6 +60,7 @@ enum OPTIONS {
 	TRACE_NAME_ONLY,
 	TRACE_NO_REMOVED,
 	TRACE_ENCL,
+	PIPE_DELETED,
 	DRYRUN,
 	SINGLERUN,
 	MAXRUNS,
@@ -128,6 +129,8 @@ UOption options[] = {
 	UOPTION_DEF_D("trace-no-removed",      0, UOPT_NO_ARG,       "does not print removed readings; implies --trace"),
 	UOPTION_DEF_D("trace-encl",            0, UOPT_NO_ARG,       "traces which enclosure pass is currently happening; implies --trace"),
 
+	UOPTION_DEF_D("deleted",               0, UOPT_NO_ARG,       "read deleted readings as such, instead of as text"),
+
 	UOPTION_DEF_D("dry-run",               0, UOPT_NO_ARG,       "make no actual changes to the input"),
 	UOPTION_DEF_D("single-run",            0, UOPT_NO_ARG,       "runs each section only once; same as --max-runs 1"),
 	UOPTION_DEF_D("max-runs",              0, UOPT_REQUIRES_ARG, "runs each section max N times; defaults to unlimited (0)"),
