@@ -1,4 +1,4 @@
-#!/usr/bin/env perl
+#!/usr/bin/perl
 use strict;
 use warnings;
 use Cwd qw(realpath);
@@ -17,7 +17,7 @@ if (!$binary || $binary eq '' || !(-x $binary)) {
 	die("Error: $binary is not executable!");
 }
 
-`"$binary" $ARGV[1] -C UTF-8 -g grammar.cg3 -I input.txt -O output.txt >stdout.txt 2>stderr.txt`;
+`"$binary" $ARGV[1] -g grammar.cg3 -I input.txt -O output.txt >stdout.txt 2>stderr.txt`;
 `diff -B expected.txt output.txt >diff.txt`;
 
 if (-s "diff.txt") {
@@ -26,8 +26,8 @@ if (-s "diff.txt") {
 	print STDERR "Success ";
 }
 
-`"$binary" $ARGV[1] -C UTF-8 -g grammar.cg3 --grammar-only --grammar-bin grammar.cg3b >stdout.bin.txt 2>stderr.bin.txt`;
-`"$binary" $ARGV[1] -C UTF-8 -g grammar.cg3b -I input.txt -O output.bin.txt >>stdout.bin.txt 2>>stderr.bin.txt`;
+`"$binary" $ARGV[1] -g grammar.cg3 --grammar-only --grammar-bin grammar.cg3b >stdout.bin.txt 2>stderr.bin.txt`;
+`"$binary" $ARGV[1] -g grammar.cg3b -I input.txt -O output.bin.txt >>stdout.bin.txt 2>>stderr.bin.txt`;
 `diff -B expected.txt output.bin.txt >diff.bin.txt`;
 
 if (-s "diff.bin.txt") {
